/*
 * main.c - Ultimeter Logger main program
 * $Id: main.c,v 1.1.1.1 2003/03/25 07:14:40 augur Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *  
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include <stdio.h>
#include<unistd.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <dlfcn.h>
#include <aprsgated/aprsgated.h>
#include <aprsgated/metric.h>
#include "xml.h"
#include "db.h"

#define _SUCCESS 0
#define _DB_ERROR 1
#define _PORT_ERROR 2
#define _CONFIG_FAIL 3

unsigned int verbose = 0;

/* Declare */
void usage(void);

int
main(int argc, char *argv[]){
	char record[1025];
	char output[1025];
	char n;
	int fd, fail = 0,  errors = 0, daemon=0, c=0;
	wxconfigPtr config = NULL; /* Configuration */
	int (*parse)(struct weather *wx, char *record); /* Parser Function Prototype */
	void *handle;
	const char *error;
	
	FILE *device;
	struct weather wx = { 0,0,0,0,0,0,0,0,0,0 };

	/* libxml2 cruft  */
	LIBXML_TEST_VERSION xmlKeepBlanksDefault (0);
	
	/* Option handler */
	while ((n = getopt (argc, argv, "dhv:f:")) != -1) {
			switch (n) {
					case 'd':
							daemon = 1;
							if (verbose & 1)
									fprintf (stderr, "Starting as a deamon...\n");
							break;
					case 'v':
							verbose = atoi (optarg);
							if (verbose & 1)
									printf ("verbose level = %d\n", verbose);
							break;
					case 'h':
							usage();
							break;
					case 'f':
							/* Configuration parser */
							config=parse_config(optarg);
							break;
			}
	}
	
	/* fail if we have no config */
	if( !config ) usage();
	if( verbose & 4 ){
		print_config(config);
	}

	/* Open Library */
	handle = dlopen (config->driver, RTLD_LAZY);
	if (!handle) {
		fputs (dlerror(), stderr);
		exit(1);
	}

	/* Attach parser function */
	parse = dlsym(handle, "parse");
	if ((error = dlerror()) != NULL)  {
		fputs(error, stderr);
		exit(1);
	}
	
	/* Daemonize */
	if (daemon == 1) {
		if ( ! daemon_start (0) ) {
			fprintf (stderr, "%s: failed to become daemon\n", argv[0]);
			return 1;
		}
	}

	/* Open device */
	fd = open_port(config->port);
	if (fd == -1) return _PORT_ERROR;
	device = fdopen(fd, "r");

    /* Connect to MySQL database */
	if (make_sql_connection(config->host, config->user, config->pass, config->database) > 0) {
		fprintf (stderr, "Could not connect to MySQL database\n");
		return _DB_ERROR;
	}

	/* Read data from device */
	/* FIXME: Move this out to it's own function */
	while( fgets( record, 1024, device ) ){
			if((verbose & 2))
				printf("Data Detected\n");

		/* If data is parsed without error */
		if(!(*parse)(&wx, record)){
			/* Insert report into database */
			weather_insert (&wx);

			/* Debugging */
			if((verbose & 2) && !(verbose &4))
				printf("Weather Data Decoded\n");
			if( verbose & 4 ){
				printf("Wind Direction: %i degrees \n", wx.wind_direction);
				printf("Wind Speed: %i KPH \n", wx.wind_speed);
				printf("Temperature: %i C\n", wx.temperature);
				printf("Rain Today: %.2f cm\n", wx.rain_today);
				printf("Humidity: %i %%\n", wx.humidity);
				printf("Barometer: %.2f millibars\n", wx.barometer);
				printf("Indoor Temperature: %i C\n", wx.in_temperature);
				printf("Indoor Humidity: %i %%\n", wx.in_humidity);
				printf("\n\n");
			}/* end if-if */
		} /* end if */
		else {
			fail++;
			if( fail++ > 1 ){
				if(verbose & 1)
					fprintf(stderr, "main(): Failed to decode data.  Resetting device port.\n");
				errors = fclose(device);
				if( errors == EOF ) return _PORT_ERROR;
				close(fd);
				if( errors == -1 ) return _PORT_ERROR;
				fd = open_port(config->port);
				if (fd == -1) return _PORT_ERROR;
				device = fdopen(fd, "r");
				fail = 0;
			} /* end else-if */
		} /* end else */
	} /* end while */

	/* clean up XML parser junk */
	xmlCleanupParser ();

	/* close database  */
	mysql_close (&mysql);
	
	/* close device */
	errors = fclose(device);
	if( errors == EOF ) return _PORT_ERROR;
	close(fd);
	if( errors == -1 ) return _PORT_ERROR;
						
	return 0;
}

void
usage(void){
	printf("aprsgated 0.1\n");
	printf("APRS Logger & Gateway, Copyright (C) 2003 David L Norris\n");
	printf("This software comes with ABSOLUTELY NO WARRANTY. This is free software,\n");
	printf("and you are welcome to modify and redistribute it under the GPL license\n");
	printf("Usage: aprsgated -f <file> [OPTIONS]\n");
	printf("  -f <file>       XML configuration file\n");
	printf("  -d              Start as daemon\n");
	printf("  -v <verbosity>  More information\n");
	printf("                  +1 - Notices\n");
	printf("                  +2 - Debug Notices\n");
	printf("                  +4 - Debug Data Structures\n");
	printf("                  Ex: -v 7 (1+2+4)\n");
	printf("  -h              Display this usage\n");
	exit(0);
}
