/*
 * serial.c - Simple serial I/O functions
 * $Id: serial.c,v 1.1.1.1 2003/03/25 07:14:40 augur Exp $
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *  
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>		/* Standard input/output definitions */
#include <string.h>		/* String function definitions */
#include <unistd.h>	/* UNIX standard function definitions */
#include <fcntl.h>		/* File control definitions */
#include <errno.h>		/* Error number definitions */
#include <termios.h>	/* POSIX terminal control definitions */

/* open_port() - Open serial port
 * Returns the file descriptor on success or -1 on error.
 */

int
open_port (char *device)
{
  int fd;
  struct termios options;

  /* Attempt to open the device  */
  fd = open (device, O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd == -1)
    {
      /* Could not open the port. */
      fprintf(stderr, "open_port: unable to open port\n");
    }
  else
    {
      fcntl (fd, F_SETFL, 0);

     /* Get the current options for the port... */
      tcgetattr (fd, &options);

      /* Set the baud rates to 2400... */
      cfsetispeed (&options, B2400);
      cfsetospeed (&options, B2400);

      /* Enable the receiver, set local mode, in canonical mode... */
      options.c_cflag |= (CLOCAL | CREAD | ICANON);

      /* Set port to 8N1  */
      options.c_cflag &= ~PARENB;
      options.c_cflag &= ~CSTOPB;
      options.c_cflag &= ~CSIZE;
      options.c_cflag |= CS8;

      /* Set the new options for the port... */
      tcsetattr (fd, TCSANOW, &options);
    }

  return (fd);
}
