#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include "xml.h"

wxconfigPtr
parse_config(char *filename)
{
  wxconfigPtr ret = NULL;
  xmlDocPtr doc;
  xmlNsPtr ns;
  xmlNodePtr cur;

  /* build XML tree from file */
  doc = xmlParseFile (filename);
  if (doc == NULL)
    return (NULL);

  /* Check the document is of the right kind */
  cur = xmlDocGetRootElement (doc);
  if (cur == NULL)
    {
      fprintf (stderr, "empty document\n");
      xmlFreeDoc (doc);
      return (NULL);
    }
  ns = xmlSearchNsByHref (doc, cur,
			  (const xmlChar *)
			  "http://www.webaugur.com/wares/aprsgated.dtd");
  if (ns == NULL)
    {
      fprintf (stderr,
	       "document of the wrong type, Correct namespace not found\n");
      xmlFreeDoc (doc);
      return (NULL);
    }

  /* allocate wxconfig struct */
  ret = (wxconfigPtr) malloc (sizeof (wxconfig));
  if (ret == NULL)
    {
      fprintf (stderr, "out of memory\n");
      return (NULL);
    }
  memset (ret, 0, sizeof (wxconfig));

  /* We don't yet care about the top level element */
  cur = cur->xmlChildrenNode;
  while (cur != NULL)
    {
		if ((!xmlStrcmp (cur->name, (const xmlChar *) "database")) && (cur->ns == ns))
			ret->database = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

		if ((!xmlStrcmp (cur->name, (const xmlChar *) "host")) && (cur->ns == ns))
			ret->host = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

		if ((!xmlStrcmp (cur->name, (const xmlChar *) "user")) && (cur->ns == ns))
			ret->user = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

		if ((!xmlStrcmp (cur->name, (const xmlChar *) "pass")) && (cur->ns == ns))
			ret->pass = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

		if ((!xmlStrcmp (cur->name, (const xmlChar *) "port")) && (cur->ns == ns))
			ret->port = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
		
		if ((!xmlStrcmp (cur->name, (const xmlChar *) "driver")) && (cur->ns == ns))
			ret->driver = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
		
		cur = cur->next;
    }
  return (ret);
}

/* debug with printf */
void
print_config (wxconfigPtr cur)
{
  if (cur == NULL)
    return;
  printf("Parsed XML Configuration File:\n");
  if (cur->database)
    printf ("	database: %s\n", cur->database);
  if (cur->host)
    printf ("	host: %s\n", cur->host);
  if (cur->user)
    printf ("	user: %s\n", cur->user);
  if (cur->pass)
	/* Don't print password to the console... */
	printf("	pass: <parsed but suppressed>\n");
    /* printf ("	pass: %s\n", cur->pass); */
  if (cur->port)
    printf ("	port: %s\n", cur->port);
  if (cur->driver)
    printf ("	driver: %s\n", cur->driver);
}

#if 0
int
main (int argc, char **argv)
{
  int i;
  wxconfigPtr cur;

  /* COMPAT: Do not genrate nodes for formatting spaces */
  LIBXML_TEST_VERSION xmlKeepBlanksDefault (0);

  for (i = 1; i < argc; i++)
    {
      cur = parse_config (argv[i]);
      if (cur)
	print_config (cur);
      else
	fprintf (stderr, "Error parsing file '%s'\n", argv[i]);

    }

  /* Clean up everything else before quitting. */
  xmlCleanupParser ();

  return (0);
}
#endif
