/* vim:ts=4:

APRS Parser with Database Support
Copyright (C) 2008 James Jefferson Jarvis	<kb0thn@aprsworld.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include <time.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <signal.h>
#include <errno.h>


#include <math.h>
#include <pwd.h>
#include "db_flatfile.h"
#include "message.h"


/* dummy database transactions */
int db_begin_txn(void){
    return 0;
}
int db_commit_txn(void){
    return 0;
}
int db_rollback_txn(void){
    return 0;
}

int make_connection (char *host, char *username, char *password, char *database) {
	return 0;
}

void close_connection(void) {
}


unsigned long packet_insert (struct raw_packet *raw) {
}


void message_insert (struct message *msg, unsigned long packet_id) {
}

void status_insert (struct status *stat, unsigned long packet_id) {
}



void laststatus_insert (struct status *stat, unsigned long packet_id) {
}


/* file format:
packet_date | %d      
latitude    | %2.6lf    
longitude   | %2.6lf
course      | %d
speed       | %d 
altitude    | %d
*/

void position_insert (struct position *pos, unsigned long packet_id) {
	FILE *fp;
	char filename[128];
	char line[1024];
	time_t now;

	if ( ! (get_insert_mask() & INSERT_POSITION) )
		return;

	/* our current time */
	now = time(NULL);

	if ( 0 == strcmp(pos->source,"") ) 
		return;

	sprintf(filename,"data/position/%s",pos->source);
//	printf("POS: %s\n",filename);

	fp=fopen(filename,"a");
        if( !fp ){
		printf("%s: %s\n", filename, strerror(errno));
		exit(1);
	}
	sprintf(line,"%externiu\t%2.6lf\t%2.6lf\t%d\t%d\t%d\n",
			(unsigned int) now,
			pos->latitude,
			pos->longitude,
			(int) pos->course_made_good,
			(int) pos->speed_over_ground,
			(int) pos->altitude
			);
	fputs(line,fp);
	fclose(fp);

}


void lastposition_insert (struct position *pos, struct status *stat, unsigned long packet_id) {
}

/*
 * Returns:
 *   hostid on success
 *   0 on failure
 *
 */
int set_qa_hostid( char *hostname ) {
	return 1;
}




void watch_trigger(char *callsign, unsigned long packet_id) {
}

void telemetry_coefficients(char *p, char *source) {
}


/*
packet_date
wind_direction
wind_speed
wind_gust
wind_sustained
temperature
rain_hour
rain_calendar_day
rain_24hour_day
humidity
barometer
luminosity
*/
void weather_insert (struct weather *wx, unsigned long packet_id) {
	FILE *fp;
	char filename[128];
	char line[1024];
	time_t now;

	if ( ! (get_insert_mask() & INSERT_WEATHER) )
		return;

	/* our current time */
	now = time(NULL);

	if ( 0 == strcmp(wx->source,"") ) 
		return;

	sprintf(filename,"data/weather/%s",wx->source);
//	printf("WX: %s\n",filename);

	fp=fopen(filename,"a");
	if( !fp ){
		printf("%s: %s\n", filename, strerror(errno));
		exit(1);
	}
	sprintf(line,"%u\t%d\t%d\t%d\t%d\t%d\t%f\t%f\t%f\t%d\t%f\t%d\t%d\t%d\t%d\t%d\n",
			(unsigned int) now,
			wx->wind_direction,
			wx->wind_speed,
			wx->wind_gust,
			wx->wind_sustained,
			wx->temperature,
			wx->rain_hour,
			wx->rain_calendar_day,
			wx->rain_24hour_day,
			wx->humidity,
			wx->barometer,
			wx->luminosity,
			wx->rain_count,
			wx->snowfall_24hour_day,
			wx->fuel_temp,
			wx->fuel_moisture
			);
	fputs(line,fp);
	fclose(fp);

}


void user_insert (char *source, char *information, unsigned long packet_id) {
}

void telemetry_insert (struct telemetry *telem, unsigned long packet_id) {
}

void telemetry_labels_units(char *p, char *source, char *table) {
}

void item_insert(struct item *itm, unsigned long packet_id) {
}


void object_insert (struct object *obj, unsigned long packet_id) {
}
