/*
APRS Parser with Database Support
Copyright (C) 2007 James Jefferson Jarvis	<kb0thn@aprsworld.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* I think this motto was in mind when APRS was designed:
 * "Never make anything simple and efficient when a way can be found to make it
 * complex and wonderful" -- Found in the fortune
 */
 
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include <time.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <signal.h>
#include <errno.h>
#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_WINSOCK_H
/* In case of insanity break glass. */
#include <winsock.h>
#endif
#include <math.h>
#include <pwd.h>
#include <mysql.h>
#include "db_mysql.h"
#include "message.h"
#include "sqlinsert.h"

int log_mysql_error() {
	openlog("APRS Parser (MySQL)",LOG_PERROR,LOG_USER);
	syslog(LOG_NOTICE,"MySQL error: %s:\n%s\n",mysql_error(&mysql), sql_buf);
	closelog();
}

/* dummy database transactions */
int db_begin_txn(void){
    return 0;
}
int db_commit_txn(void){
    return 0;
}
int db_rollback_txn(void){
    return 0;
}

int make_connection (char *host, char *username, char *password, char *database) {
	mysql_init (&mysql);

	if ( NULL == mysql_real_connect (&mysql, host, username, password, database, (char *) NULL, 0, (char *) NULL)) {
		if (get_verbose() & VERBOSE_ERRORS) {
			fprintf (stderr, "Failed to connect to database: Error: %s\n", mysql_error (&mysql));
		}
		return 1;
	}

	return 0;
}

void close_connection(void) {
    mysql_close (&mysql);
}


unsigned long packet_insert (struct raw_packet *raw) {
	struct sql_insert sql_query;
	unsigned long packet_id = 1;
	int newlen;

	if ( ! (get_insert_mask() & INSERT_RAW) )
		return 0;

	sql_insert_table(&sql_query,"raw");

	sql_insert_column(&sql_query,"packet_time","-1");
	sql_insert_column(&sql_query,"source","-1");
	sql_insert_column(&sql_query,"destination","-1");
	sql_insert_column(&sql_query,"information","");
	sql_insert_column(&sql_query,"ip_source","");
	sql_insert_column(&sql_query,"digipeater_0","");
	sql_insert_column(&sql_query,"digipeater_1","");
	sql_insert_column(&sql_query,"digipeater_2","");
	sql_insert_column(&sql_query,"digipeater_3","");
	sql_insert_column(&sql_query,"digipeater_4","");
	sql_insert_column(&sql_query,"digipeater_5","");
	sql_insert_column(&sql_query,"digipeater_6","");
	sql_insert_column(&sql_query,"digipeater_7","");
       	sql_insert_column(&sql_query,"hostid","");
	sql_insert_column(&sql_query,"","");

	snprintf(sql_buf,256,"unix_timestamp(now())");
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->source);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->destination);
	sql_insert_value(&sql_query, sql_buf);

	/* insert raw information */
	if ( get_insert_information() ) {
		snprintf(sql_buf,256,"%s",raw->information);
	} else {
		strcpy(sql_buf,"");
	}
	sql_insert_value(&sql_query, sql_buf);

	snprintf(sql_buf,256,"%s",raw->ip_source);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[0]);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[1]);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[2]);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[3]);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[4]);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[5]);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[6]);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",raw->digipeater[7]);
	sql_insert_value(&sql_query, sql_buf);
        snprintf(sql_buf,256,"%d",qa_hostid);
       	sql_insert_value(&sql_query, sql_buf);
	
	sql_insert_value(&sql_query,"");

	//print_query(&sql_query);

	query(&sql_query,sql_buf,"INSERT");

	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);
	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();

	return mysql_insert_id(&mysql);
}



void message_insert (struct message *msg, unsigned long packet_id) {
	struct sql_insert sql_query;
	int newlen;

	if ( ! ( get_insert_mask() & INSERT_MESSAGE ) )
		return;

	sql_insert_table(&sql_query,"messages");

	sql_insert_column(&sql_query,"packet_id","-1");
	sql_insert_column(&sql_query,"source","");
	sql_insert_column(&sql_query,"packet_date","");
	sql_insert_column(&sql_query,"addressee","");
	sql_insert_column(&sql_query,"text","");
	sql_insert_column(&sql_query,"id","");
	sql_insert_column(&sql_query,"","");

	snprintf(sql_buf,256,"%lu",packet_id);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",msg->source);
	sql_insert_value(&sql_query, sql_buf);

	snprintf(sql_buf,256,"now()");
	sql_insert_value(&sql_query, sql_buf);

	snprintf(sql_buf,256,"%s",msg->addressee);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",msg->text);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",msg->id);
	sql_insert_value(&sql_query, sql_buf);

	sql_insert_value(&sql_query,"");

	query(&sql_query,sql_buf,"INSERT");

	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);

	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();

}

void status_insert (struct status *stat, unsigned long packet_id) {
	struct sql_insert sql_query;
	int newlen;

	if ( ! (get_insert_mask() & INSERT_STATUS) ) 
		return;

	/* insert into the last status table */
	laststatus_insert(stat, packet_id);
	
	sql_insert_table(&sql_query,"status");

	sql_insert_column(&sql_query,"packet_id","-1");
	sql_insert_column(&sql_query,"source","");
	sql_insert_column(&sql_query,"power","0");
	sql_insert_column(&sql_query,"height","0");
	sql_insert_column(&sql_query,"gain","0.000000");
	sql_insert_column(&sql_query,"directivity","-1");
	sql_insert_column(&sql_query,"rate","0");
	sql_insert_column(&sql_query,"comment","");
	sql_insert_column(&sql_query,"symbol_table","");
	sql_insert_column(&sql_query,"symbol_code","");
	sql_insert_column(&sql_query,"","");

	snprintf(sql_buf,256,"%lu",packet_id);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",stat->source);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->power);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->height);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%lf",stat->gain);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->directivity);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->rate);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",stat->comment);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%c",stat->symbol_table);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%c",stat->symbol_code);
	sql_insert_value(&sql_query, sql_buf);

	sql_insert_value(&sql_query,"");

	query(&sql_query,sql_buf,"INSERT");

	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);

	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();
}



void laststatus_insert (struct status *stat, unsigned long packet_id) {
	struct sql_insert sql_query;
	int newlen;

	if ( ! (get_insert_mask() & INSERT_LAST) ) 
		return;

	sql_insert_table(&sql_query,"laststatus");

	sql_insert_column(&sql_query,"packet_id","-1");
	sql_insert_column(&sql_query,"packet_date","");
	sql_insert_column(&sql_query,"source","");
	sql_insert_column(&sql_query,"power","0");
	sql_insert_column(&sql_query,"height","0");
	sql_insert_column(&sql_query,"gain","0.000000");
	sql_insert_column(&sql_query,"directivity","-1");
	sql_insert_column(&sql_query,"rate","0");
	sql_insert_column(&sql_query,"comment","");
	sql_insert_column(&sql_query,"symbol_table","");
	sql_insert_column(&sql_query,"symbol_code","");
	sql_insert_column(&sql_query,"","");

	snprintf(sql_buf,256,"%lu",packet_id);
	sql_insert_value(&sql_query, sql_buf);
	
	snprintf(sql_buf,256,"now()");
	sql_insert_value(&sql_query, sql_buf);
	
	snprintf(sql_buf,256,"%s",stat->source);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->power);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->height);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%lf",stat->gain);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->directivity);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%d",stat->rate);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%s",stat->comment);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%c",stat->symbol_table);
	sql_insert_value(&sql_query, sql_buf);
	snprintf(sql_buf,256,"%c",stat->symbol_code);
	sql_insert_value(&sql_query, sql_buf);

	sql_insert_value(&sql_query,"");

	//print_query(&sql_query);

	query(&sql_query,sql_buf,"REPLACE");

	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);
	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();

}



void position_insert (struct position *pos, unsigned long packet_id) {
	struct sql_insert sql_query;
	char *quotebuf[1024];
	size_t newlen;

	if ( ! (get_insert_mask() & INSERT_POSITION) )
		return;

	/* We don't even want to bother inserting if the position looks to be obviously bogus.
	* This gets rid of unlocked GPSs, unconfigured APRS clients, and the like.  
 	*/ 

	if ( pos->latitude != 0 && pos->longitude != 0 ) {	

		sql_insert_table(&sql_query,"position");
		
		sql_insert_column(&sql_query,"packet_id","-1");
		sql_insert_column(&sql_query,"source","");
		sql_insert_column(&sql_query,"packet_date","");
		sql_insert_column(&sql_query,"time_of_fix","0");
		sql_insert_column(&sql_query,"latitude","0.000000");
		sql_insert_column(&sql_query,"longitude","0.000000");
		sql_insert_column(&sql_query,"course","-1.000000");
		sql_insert_column(&sql_query,"speed","-1.000000");
		sql_insert_column(&sql_query,"altitude","0.000000");
		sql_insert_column(&sql_query,"","");

		snprintf(sql_buf,256,"%lu",packet_id);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%s",pos->source);
		sql_insert_value(&sql_query, sql_buf);
	
		snprintf(sql_buf,256,"now()");
		sql_insert_value(&sql_query, sql_buf);
		
		snprintf(sql_buf,256,"%ld",pos->time_of_fix);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->latitude);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->longitude);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->course_made_good);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->speed_over_ground);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->altitude);
		sql_insert_value(&sql_query, sql_buf);

		sql_insert_value(&sql_query,"");

		//print_query(&sql_query);

		query(&sql_query,sql_buf,"INSERT");

		if (get_verbose() & VERBOSE_SQL)
			printf ("%s;\n", sql_buf);

		if ( mysql_query (&mysql, sql_buf) )
			log_mysql_error();
	}
}


void lastposition_insert (struct position *pos, struct status *stat, unsigned long packet_id) {
	struct sql_insert sql_query;
	int newlen, count;

	if ( ! (get_insert_mask() & INSERT_LAST) ) 
		return;

	/* We don't even want to bother inserting if the position looks to be obviously bogus.
 	* This gets rid of unlocked GPSs, unconfigured APRS clients, and the like.  
 	*/ 

	if ( pos->latitude != 0 && pos->longitude != 0 ) {	
		sql_insert_table(&sql_query,"lastposition");
		
		sql_insert_column(&sql_query,"packet_id","-1");
		sql_insert_column(&sql_query,"packet_date","-1");
		sql_insert_column(&sql_query,"source","");
		sql_insert_column(&sql_query,"latitude","0.000000");
		sql_insert_column(&sql_query,"longitude","0.000000");
		sql_insert_column(&sql_query,"course","-1.000000");
		sql_insert_column(&sql_query,"speed","-1.000000");
		sql_insert_column(&sql_query,"altitude","0.000000");
		sql_insert_column(&sql_query,"status","");
		sql_insert_column(&sql_query,"symbol_table","");
		sql_insert_column(&sql_query,"symbol_code","");
		sql_insert_column(&sql_query,"","");

		snprintf(sql_buf,256,"%lu",packet_id);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"now()");
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%s",pos->source);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->latitude);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->longitude);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->course_made_good);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->speed_over_ground);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%f",pos->altitude);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%s",stat->comment);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%c",stat->symbol_table);
		sql_insert_value(&sql_query, sql_buf);
		snprintf(sql_buf,256,"%c",stat->symbol_code);
		sql_insert_value(&sql_query, sql_buf);

		sql_insert_value(&sql_query,"");

		//print_query(&sql_query);

		query(&sql_query,sql_buf,"REPLACE");

		if (get_verbose() & VERBOSE_SQL)
			printf ("%s;\n", sql_buf);

		if ( mysql_query (&mysql, sql_buf) )
			log_mysql_error();
	}
}

/*
 * Returns:
 *   hostid on success
 *   0 on failure
 *
 */
int set_qa_hostid( char *hostname ) {
	int retval = 0;
	int assign_new_host=0;
	int newlen;
	
    sprintf(sql_buf, "SELECT * FROM qa_hosts WHERE host='%s' LIMIT 1;", hostname);

    if ( mysql_query (&mysql, sql_buf) ) {
        log_mysql_error();
        return 0;
    } else {
        result = mysql_store_result (&mysql);
        row = mysql_fetch_row (result);
    
        if( !row ) {
            assign_new_host=1;
            retval = 0;
        } else {  // Let's not explode in case we missed something.
            retval = atoi(row[0]);
            if (get_verbose() & VERBOSE_DEBUG)
                printf ("Using qa_hosts id %s for %s\n", row[0], hostname);
        }
        mysql_free_result (result);
    }

    if ( assign_new_host ) {
        sprintf(sql_buf,"INSERT INTO qa_hosts (host) VALUES (\"%s\")",hostname);
        printf("Query: %s\n",sql_buf);
        if ( mysql_query (&mysql, sql_buf) ) {
            log_mysql_error();
            return 0;
        }
        retval = set_qa_hostid(hostname); // recursively call ourself to get the new id
    } 

    return retval;

}




void watch_trigger(char *callsign, unsigned long packet_id) {
	char escaped[20];	/* callsign can be 10 characters, worst case every character is escaped */
	char query[256];
	int pid;
	char *arguments[3];

    mysql_escape_string(escaped,callsign,strlen(callsign));

    /* do an equ-join to find the program we want to execute */
    sprintf(query,"SELECT p.directory, p.command, p.user FROM watch_programs p, watch_users u WHERE u.source = \"%s\" AND p.program = u.program",escaped);

    if ( mysql_query (&mysql, query) )
        log_mysql_error();

    result = mysql_use_result (&mysql);
    while ( row = mysql_fetch_row (result) ) {
        if ( get_verbose() & VERBOSE_SQL )
            printf("%s has triggered: %s\n",callsign,row[1]);
    
        /* Before executing we need to fork(), setuid(), change to new working directory
         * then exec program with argv[1] = packet_id */

        pid = fork();
        if ( pid == 0 ) {
            struct passwd *pwd;
            if ( get_verbose() & VERBOSE_SQL )
                printf("\tattempting to run %s as %s in %s (pid=%d)\n",row[1],row[2],row[0],(int) getpid());

            if ( row[2][0] ) {	/* If user isn't specified, then we don't setuid() */
                if ( (pwd = getpwnam(row[2])) == NULL ) {
                    openlog("APRS Parser (MySQL)",LOG_PERROR,LOG_USER);
                    syslog(LOG_NOTICE,"Cannot execute trigger because username %s not found\n",row[2]);
                    closelog();
                    exit(1);	/* exit, not return, because we are a new process */
                }

                if ( setuid(pwd->pw_uid) ) {
                    openlog("APRS Parser (MySQL)",LOG_PERROR,LOG_USER);
                    syslog(LOG_NOTICE,"Cannot execute trigger: setuid failed\n");
                    closelog();
                    exit(2);	/* exit, not return, because we are a new process */
                }
            }

            if ( strncmp(row[0],"",1) && chdir(row[0]) ) {
                openlog("APRS Parser (MySQL)",LOG_PERROR,LOG_USER);
                syslog(LOG_NOTICE,"Cannot execute trigger because %s not found\n",row[0]);
                closelog();
                exit(3);	/* exit, not return, because we are a new process */
            }

            arguments[0] = row[1];
            sprintf(query,"%lu",packet_id);
            arguments[1] = query;
            arguments[2] = (char *) 0;

            execv(arguments[0],arguments);
            exit(0); // we are forked, so we exit when we are done executing
        }

    }

    mysql_free_result(result);

}

/* should handle this INSERT in a way that doesn't duplicate coefficients calc */
void telemetry_coefficients(char *p, char *source) {
	int i;
	char *val;
	int newlen;

	double a[5];
	double b[5];
	double c[5];

	for ( i=0 ; i<5 ; i++ ) {
		a[i]=0;
		b[i]=1;
		c[i]=0;
	}

	if ( strlen(p) < 5 )
		return;

	p += 5;	// swallow the EQNS.

	i=0;
	val = strtok(p,",");
	do {
		if ( 0 == val )
			break;
		a[i] = atof(val);
		val = strtok(NULL,",");
		if ( 0 == val )
			break;
		b[i] = atof(val);
		val = strtok(NULL,",");
		if ( 0 == val )
			break;
		c[i] = atof(val);

		i++;
		val = strtok(NULL,",");
	} while ( i<5 );

	if ( get_verbose() & VERBOSE_DEBUG ) {
		printf("Telemetry coefficients:\n");
		for ( i=0 ; i<5 ; i++ ) {
			printf("\t[%d] a=%lf b=%lf c=%lf\n",i,a[i],b[i],c[i]);
		}
	}

		snprintf(sql_buf, 512,"INSERT INTO telemetry_coefficients VALUES ('%s',now(), %lf,%lf,%lf, %lf,%lf,%lf, %lf,%lf,%lf, %lf,%lf,%lf, %lf,%lf,%lf )",
				source,
				a[0],b[0],c[0],
				a[1],b[1],c[1],
				a[2],b[2],c[2],
				a[3],b[3],c[3],
				a[4],b[4],c[4]);

		if ( get_verbose() & VERBOSE_SQL )
			printf("SQL: %sql_buf\n", sql_buf);

		if ( mysql_query (&mysql, sql_buf) )
			log_mysql_error();
}


void weather_insert (struct weather *wx, unsigned long packet_id) {
	struct sql_insert sql_query;
	int newlen, count;

	if ( ! (get_insert_mask() & INSERT_WEATHER) ) 
		return;

	    sql_insert_table(&sql_query,"weather");

	    sql_insert_column(&sql_query,"packet_id","-1");
	    sql_insert_column(&sql_query,"source","");
	    sql_insert_column(&sql_query,"packet_date","");
	    sql_insert_column(&sql_query,"wind_direction","-1");
	    sql_insert_column(&sql_query,"wind_speed","-1");
	    sql_insert_column(&sql_query,"wind_gust","-1");
	    sql_insert_column(&sql_query,"wind_sustained","-1");
	    sql_insert_column(&sql_query,"temperature","-100");
	    sql_insert_column(&sql_query,"rain_hour","-1.000000");
	    sql_insert_column(&sql_query,"rain_calendar_day","-1.000000");
	    sql_insert_column(&sql_query,"rain_24hour_day","-1.000000");
	    sql_insert_column(&sql_query,"humidity","0");
	    sql_insert_column(&sql_query,"barometer","-1.000000");
	    sql_insert_column(&sql_query,"luminosity","-1");
	    sql_insert_column(&sql_query,"","");

	    snprintf(sql_buf,256,"%lu",packet_id);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%s",wx->source);
	    sql_insert_value(&sql_query, sql_buf);

	    snprintf(sql_buf,256,"now()");
	    sql_insert_value(&sql_query, sql_buf);

	    snprintf(sql_buf,256,"%d",wx->wind_direction);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%d",wx->wind_speed);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%d",wx->wind_gust);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%d",wx->wind_sustained);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%d",wx->temperature);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%f",wx->rain_hour);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%f",wx->rain_calendar_day);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%f",wx->rain_24hour_day);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%d",wx->humidity);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%f",wx->barometer);
	    sql_insert_value(&sql_query, sql_buf);
	    snprintf(sql_buf,256,"%d",wx->luminosity);
	    sql_insert_value(&sql_query, sql_buf);

	    sql_insert_value(&sql_query,"");

	    query(&sql_query,sql_buf,"INSERT");

	    if (get_verbose() & VERBOSE_SQL)
	        printf ("%s;\n", sql_buf);

	    if ( mysql_query (&mysql, sql_buf) )
	        log_mysql_error();


	    sql_insert_table(&sql_query,"lastweather");
	    query(&sql_query,sql_buf,"REPLACE");

	    if (get_verbose() & VERBOSE_SQL)
	        printf ("%s;\n", sql_buf);

	    if ( mysql_query (&mysql, sql_buf) )
	        log_mysql_error();

}

void last_user_insert (char *source, char *information, unsigned long packet_id) {
	int i;
	struct sql_insert sql_query;
	int newlen;

	if ( ! (get_insert_mask()&INSERT_LAST) ) 
		return;

	sql_insert_table(&sql_query,"lastuserdefined");
	
    	sql_insert_column(&sql_query,"packet_id","-1");
    	sql_insert_column(&sql_query,"source","");
    	sql_insert_column(&sql_query,"packet_date","");
    	sql_insert_column(&sql_query,"user_id","");
    	sql_insert_column(&sql_query,"packet_type","");
    	sql_insert_column(&sql_query,"information","");
    	sql_insert_column(&sql_query,"","");
	
    	snprintf(sql_buf,256,"%lu",packet_id);
    	sql_insert_value(&sql_query,sql_buf);
    	snprintf(sql_buf,256,"%s",source);
    	sql_insert_value(&sql_query,sql_buf);
	
    	snprintf(sql_buf,256,"now()");
    	sql_insert_value(&sql_query,sql_buf);
	
	/* user id and packet type */
    	snprintf(sql_buf,256,"%c",information[1]);
    	sql_insert_value(&sql_query,sql_buf);
    	snprintf(sql_buf,256,"%c",information[2]);
    	sql_insert_value(&sql_query,sql_buf);
	
    	snprintf(sql_buf,256,"%s",information);
    	sql_insert_value(&sql_query,sql_buf);
	
    	sql_insert_value(&sql_query,"");
	
    	query(&sql_query,sql_buf,"REPLACE");
	
    	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);
	
    	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();
}

void user_insert (char *source, char *information, unsigned long packet_id) {
	int i;
	struct sql_insert sql_query;
	int newlen;

	last_user_insert(source,information,packet_id);

	if ( ! (get_insert_mask()&INSERT_USER) )
		return;
	
	sql_insert_table(&sql_query,"userdefined");
	
    	sql_insert_column(&sql_query,"packet_id","-1");
    	sql_insert_column(&sql_query,"source","");
    	sql_insert_column(&sql_query,"packet_date","");
    	sql_insert_column(&sql_query,"user_id","");
    	sql_insert_column(&sql_query,"packet_type","");
    	sql_insert_column(&sql_query,"information","");
    	sql_insert_column(&sql_query,"","");
	
    	snprintf(sql_buf,256,"%lu",packet_id);
    	sql_insert_value(&sql_query,sql_buf);
    	snprintf(sql_buf,256,"%s",source);
    	sql_insert_value(&sql_query,sql_buf);
	
    	snprintf(sql_buf,256,"now()");
    	sql_insert_value(&sql_query,sql_buf);
	
	/* user id and packet type */
    	snprintf(sql_buf,256,"%c",information[1]);
    	sql_insert_value(&sql_query,sql_buf);
    	snprintf(sql_buf,256,"%c",information[2]);
    	sql_insert_value(&sql_query,sql_buf);
	
    	snprintf(sql_buf,256,"%s",information);
    	sql_insert_value(&sql_query,sql_buf);
	
    	sql_insert_value(&sql_query,"");
	
    	query(&sql_query,sql_buf,"INSERT");
	
    	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);
	
    	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();
}

void telemetry_insert (struct telemetry *telem, unsigned long packet_id) {
	int i;
	struct sql_insert sql_query;
	int newlen;

	if ( ! (get_insert_mask()&INSERT_TELEM) ) 
		return;

	sql_insert_table(&sql_query,"telemetry");
	
    	sql_insert_column(&sql_query,"packet_id","-1");
    	sql_insert_column(&sql_query,"source","");
    	sql_insert_column(&sql_query,"packet_date","");
    	sql_insert_column(&sql_query,"sequence","-1");
    	sql_insert_column(&sql_query,"analog_0","");
    	sql_insert_column(&sql_query,"analog_1","");
    	sql_insert_column(&sql_query,"analog_2","");
    	sql_insert_column(&sql_query,"analog_3","");
    	sql_insert_column(&sql_query,"analog_4","");
    	sql_insert_column(&sql_query,"digital","");
    	sql_insert_column(&sql_query,"","");
	
    	snprintf(sql_buf,256,"%lu",packet_id);
    	sql_insert_value(&sql_query,sql_buf);
    	snprintf(sql_buf,256,"%s",telem->source);
    	sql_insert_value(&sql_query,sql_buf);
	
    	snprintf(sql_buf,256,"now()");
    	sql_insert_value(&sql_query,sql_buf);
	
    	snprintf(sql_buf,256,"%d",telem->sequence);
    	sql_insert_value(&sql_query,sql_buf);
	
    	for ( i=0 ; i<5 ; i++ ) {
		snprintf(sql_buf,256,"%d",telem->analog[i]);
		sql_insert_value(&sql_query,sql_buf);
    	}
	
    	snprintf(sql_buf,256,"%d",telem->digital);
    	sql_insert_value(&sql_query,sql_buf);
	
    	sql_insert_value(&sql_query,"");
	
    	query(&sql_query,sql_buf,"INSERT");
	
    	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);
	
    	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();
}

void telemetry_labels_units(char *p, char *source, char *table) {
	/* UNIT.volt,deg.C,None,None,None,high,high,hig,hig,hig,hi,hi,hi */
	int i;
	char *val;
	struct sql_insert sql_query;
	int newlen;
	char *tmptok;

	char a_0[128];
	char a_1[128];
	char a_2[128];
	char a_3[128];
	char a_4[128];

	char d_0[128];
	char d_1[128];
	char d_2[128];
	char d_3[128];
	char d_4[128];
	char d_5[128];
	char d_6[128];
	char d_7[128];

	if ( !(get_insert_mask()&INSERT_TELEM) ) 
		return;

	sql_insert_table(&sql_query,table);

	sql_insert_column(&sql_query,"source","");
	sql_insert_column(&sql_query,"packet_date","");
	/* analog labels */
	sql_insert_column(&sql_query,"a_0","");
	sql_insert_column(&sql_query,"a_1","");
	sql_insert_column(&sql_query,"a_2","");
	sql_insert_column(&sql_query,"a_3","");
	sql_insert_column(&sql_query,"a_4","");
	/* digital labels */
	sql_insert_column(&sql_query,"d_0","");
	sql_insert_column(&sql_query,"d_1","");
	sql_insert_column(&sql_query,"d_2","");
	sql_insert_column(&sql_query,"d_3","");
	sql_insert_column(&sql_query,"d_4","");
	sql_insert_column(&sql_query,"d_5","");
	sql_insert_column(&sql_query,"d_6","");
	sql_insert_column(&sql_query,"d_7","");
	sql_insert_column(&sql_query,"","");

	/* packet source and date */
	snprintf(sql_buf, 256,"%s",source);

	sql_insert_value(&sql_query, sql_buf);

	snprintf(sql_buf, 256,"now()");

	sql_insert_value(&sql_query, sql_buf);
	
	if ( get_verbose() & VERBOSE_SQL )
		printf("%sql_buf\n",table);

	if ( strlen(p) < 5 )
		return;

	/* skip the identifier */
	p += 5;

	snprintf(sql_buf, 256, "%s",strtok(p,","));
	sql_insert_value(&sql_query, sql_buf);

	for ( i=0 ; i < 13 ; i++ ) {
		snprintf(sql_buf, 256,"%s",strtok(NULL,","));
		sql_insert_value(&sql_query, sql_buf);
	}


	sql_insert_value(&sql_query,"");
	
	query(&sql_query, sql_buf, "INSERT");

	if (get_verbose() & VERBOSE_SQL)
		printf ("%s;\n", sql_buf);

	if ( mysql_query (&mysql, sql_buf) )
		log_mysql_error();
}

void item_insert(struct item *itm, unsigned long packet_id) {
    /* Not implemented for MySQL */
}


void object_insert (struct object *obj, unsigned long packet_id) {

	if ( obj->kill ) {
		if ( get_verbose() & VERBOSE_SQL ) {
			printf("---> Killing object %s\n",obj->name);
	
			printf("DELETE FROM lastposition WHERE source='%s'\n",obj->name);
			printf("DELETE FROM laststatus WHERE source='%s'\n",obj->name);
			printf("DELETE FROM lastweather WHERE source='%s'\n",obj->name);
		}
		
		return;
	} 

	/* we want it to show up in lastposition by name */
	strcpy(obj->pos.source,obj->name);

	/* do the insert */
	lastposition_insert( &obj->pos, &obj->stat, packet_id);
	position_insert( &obj->pos, packet_id);
	/* BUG: we need to handle weather */
	
	/* mark when the housekeeper should expire it */
	// printf("REPLACE INTO object_expires (source,packet_id,posted) VALUES ('%s',%lu,now())\n",obj->name,packet_id);

}
