#include <stdio.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* Example MIC-E packets as captured from goldeneye 

aprs: fm KB0THN-7 to T4PR7V via WIDE WIDE ctl UIv pid=F0(Text) len 15
0000  'wB`l Rk/]"6:}.

aprs: fm W0STV-14 to TTPQ1X via W0MXW-1* WIDE* WIDE ctl UIv pid=F0(Text) len 15
0000  'x7Xl!Yk/]"7P}.

Destination field:
	6 latitude digits
	3-bit Mic-E message identifier
	The North/South and West/East indications
	The longitude offset indication

Information field:
	Encoded longitude
	Encoded course and speed
	Symbol table and symbol code
	(Optional) field containing either telemetry data or status spring.
	(Optional) altitude

*/

int d28( char c ) {
	if ( c < '&' || c > 'a' ) {
		printf("Character is not d28 encoded\n");
		return -1;
	}
	return c-28;
}

void parse_mic_e( char *destination, char *information ) {

	/* This is esentially a lookup table. Do not change the white spaces */
	static char dest[] = 	"0123456789       0123456789     0123456789 ";
	/*								 0123456789       ABCDEFGHIJKLMNOPQRSTUVWXYZ */
	static char *standard_msg[] = {
		"EMERGENCY",
		"PRIORITY",
		"Special",
		"Committed",
		"Returning",
		"In Service",
		"Enroute",
		"Off duty"
	};
	static char *custom_msg[] = {
		"EMERGENCY",
		"Custom-6",
		"Custom-5",
		"Custom-4",
		"Custom-3",
		"Custom-2",
		"Custom-1",
		"Custom-0"
	};

	double latitude;
	double longitude;
	int course;
	int speed;
	int altitude;
	int msg;

	char buffer[16];

	int i,j;

	i=j=0;

	printf("Destination: %s\n",destination);
	printf("Information: %s\n",information);


	/* Destination address has to be six characters, information has to be nine */
	if ( strlen(destination) < 6 || strlen(information) < 9 )
		return;

	for (i='0' ; i<='Z' ; i++ )
		printf("table: \"%c\"\n",dest[(i - '0')]);
	
	/* check for southern hemisphere (byte 3) */
	if ( destination[3] >= '0' && destination[3] <= 9 ) {
		printf("Southern Hemisphere ");
		buffer[j++] = '-';
	} else {
		printf("Northern Hemisphere ");
	}

	/* check for longitude offset (byte 4) */
	if ( destination[4] >= 'P' && destination[4] <= 'Z' ) {
		printf("longitude = 100;");
		longitude=100;
	} else {
		longitude=0;
	}

	/* check for eastern hemisphere (byte 5) */
	if ( destination[5] >= '0' && destination[5] <= 9 ) {
		printf("Eastern Hemisphere ");
		buffer[j++] = '-';
	} else {
		printf("Western Hemisphere ");

	for ( i=0 ; i<6 ; i++ ) {
		if ( 2 == i )
			buffer[j++] = '.';
		buffer[j++] = dest[(destination[i]-'0')];
	}
	buffer[j]='\0';

	printf("Latitude: %s\n",buffer);

	/* BUG BUG BUG - need to make this actually do something. See table on page 45 */
	for ( i=0 ; i<3 ; i++ ) {
		if ( destination[i] >= '0' && destination[i] <= '9' ) {
			/* zero */

		} else if ( destination[i] >= 'A' && destination[i] <= 'K' ) {
			/* 1 (custom) */

		} else if ( destination[i] >= 'P' && destination[i] <= 'Z' ) {
			/* 1 (standard) */

		} else if ( destination[i] == 'L' ) {
			/* zero */

		}
	}

	/* BUG check the Information Field Data (page 46) and handle the Kenwood exception */



}


int main(void) {

	parse_mic_e("T4PR7V","'wB`l Rk/]\"6:}.");


	return 0;
}
