-- AprsWorld PostgreSQL/PostGIS schema
--
-- PostgreSQL and PostGIS required
-- Tested on PostgreSQL 8.2.5 with PostGIS 1.2.1
--
-- 
-- Performance notes:
-- 
-- As a cron job, run below every hour or so:
--
-- BEGIN;
-- LOCK aprs_lastposition IN ACCESS EXCLUSIVE MODE;
-- LOCK aprs_lastobject IN ACCESS EXCLUSIVE MODE;
-- LOCK aprs_laststatus IN ACCESS EXCLUSIVE MODE;
-- LOCK aprs_lastweather IN ACCESS EXCLUSIVE MODE;
-- LOCK aprs_lastitem IN ACCESS EXCLUSIVE MODE;
-- LOCK aprs_lastuserdefined IN ACCESS EXCLUSIVE MODE;
-- CLUSTER aprs_lastposition;
-- CLUSTER aprs_lastobject;
-- CLUSTER aprs_laststatus;
-- CLUSTER aprs_lastweather;
-- CLUSTER aprs_lastitem;
-- CLUSTER aprs_lastuserdefined;
-- COMMIT;
--
--
-- As another cron job, run below every night or so:
--
-- delete from aprs_lastitem where packet_date < now() - '1 day'::interval;
-- delete from aprs_lastobject where packet_date < now() - '1 day'::interval;
-- delete from aprs_lastposition where packet_date < now() - '1 day'::interval;
-- delete from aprs_lastuserdefined where packet_date < now() - '1 day'::interval;
-- delete from aprs_lastweather where packet_date < now() - '1 day'::interval;
-- vacuum aprs_lastitem;
-- vacuum aprs_lastobject;
-- vacuum aprs_lastposition;
-- vacuum aprs_lastuserdefined;
-- vacuum aprs_lastweather;
-- analyze;


--
-- aprs_lastposition
--
DROP INDEX aprs_lastposition_id_idx;
--DROP INDEX aprs_lastposition_geom_idx;

SELECT DropGeometryColumn('', 'aprs_lastposition', 'position');
DROP TABLE aprs_lastposition;
CREATE TABLE aprs_lastposition (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  packet_date timestamp NOT NULL DEFAULT now(),
  source varchar(20) NOT NULL default '',
  course int default NULL,
  speed int default NULL,
  altitude int default NULL,
  status varchar(256),
  symbol_table char(1) default NULL,
  symbol_code char(1) default NULL,
  PRIMARY KEY (source)
);
SELECT AddGeometryColumn('', 'aprs_lastposition', 'position', -1, 'POINT', 2 );

CREATE INDEX aprs_lastposition_id_idx ON aprs_lastposition(packet_id);
--CREATE INDEX aprs_lastposition_geom_idx ON aprs_lastposition USING GIST (position GIST_GEOMETRY_OPS);

CLUSTER aprs_lastposition_pkey ON aprs_lastposition;


--
-- aprs_position
--

DROP INDEX aprs_position_ts_idx;
--DROP INDEX aprs_position_geom_idx;
SELECT DropGeometryColumn('', 'aprs_position', 'position');
DROP TABLE aprs_position;

CREATE TABLE aprs_position (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  source varchar(20) NOT NULL default '',
  packet_date timestamp not null default now(),
  time_of_fix int default NULL,
  course int default NULL,
  speed int default NULL,
  altitude int default NULL
-- ,PRIMARY KEY (packet_id)
);

SELECT AddGeometryColumn('', 'aprs_position', 'position', -1, 'POINT', 2 );
--CREATE INDEX aprs_position_geom_idx ON aprs_position USING GIST (position GIST_GEOMETRY_OPS);
CREATE INDEX aprs_position_ts_idx ON aprs_position (packet_date);

--
-- aprs_object
--

DROP INDEX aprs_object_ts_idx;
DROP INDEX aprs_object_source_idx;
--DROP INDEX aprs_object_geom_idx;
SELECT DropGeometryColumn('', 'aprs_object', 'position');
DROP TABLE aprs_object;

CREATE TABLE aprs_object (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  packet_date timestamp not null default now(),
  source varchar(20) NOT NULL,
  name varchar(20) NOT NULL,
  course int default NULL,
  speed int default NULL,
  altitude int default NULL,
  symbol_table char(1) default NULL,
  symbol_code char(1) default NULL,
  comment varchar(256),
  wind_direction int default NULL,
  wind_speed int default NULL,
  wind_gust int default NULL,
  wind_sustained int default NULL,
  temperature int default NULL,
  rain_hour float default NULL,
  rain_calendar_day float default NULL,
  rain_24hour_day float default NULL,
  humidity int default NULL,
  barometer real default NULL,
  luminosity int default NULL
--  ,PRIMARY KEY (packet_id)
);

SELECT AddGeometryColumn('', 'aprs_object', 'position', -1, 'POINT', 2 );
--CREATE INDEX aprs_object_geom_idx ON aprs_object USING GIST (position GIST_GEOMETRY_OPS);
CREATE INDEX aprs_object_source_idx ON aprs_object  (source);
CREATE INDEX aprs_object_ts_idx ON aprs_object (packet_date);

--
-- aprs_lastobject
--

DROP INDEX aprs_lastobject_ts_idx;
DROP INDEX aprs_lastobject_source_idx;
--DROP INDEX aprs_lastobject_geom_idx;
SELECT DropGeometryColumn('', 'aprs_lastobject', 'position');
DROP TABLE aprs_lastobject;

CREATE TABLE aprs_lastobject (
  packet_id BIGINT NOT NULL default currval('aprs_raw_seq'),
  packet_date timestamp not null default now(),
  source varchar(20) NOT NULL,
  name varchar(20) NOT NULL,
  course int default NULL,
  speed int default NULL,
  altitude int default NULL,
  symbol_table char(1) default NULL,
  symbol_code char(1) default NULL,
  comment varchar(256),
  wind_direction int default NULL,
  wind_speed int default NULL,
  wind_gust int default NULL,
  wind_sustained int default NULL,
  temperature int default NULL,
  rain_hour float default NULL,
  rain_calendar_day float default NULL,
  rain_24hour_day float default NULL,
  humidity int default NULL,
  barometer real default NULL,
  luminosity int default NULL,
  PRIMARY KEY (source, name)
);

SELECT AddGeometryColumn('', 'aprs_lastobject', 'position', -1, 'POINT', 2 );
--CREATE INDEX aprs_lastobject_geom_idx ON aprs_lastobject USING GIST (position GIST_GEOMETRY_OPS);
CREATE INDEX aprs_lastobject_source_idx ON aprs_lastobject (source);
CREATE INDEX aprs_lastobject_ts_idx ON aprs_lastobject (packet_date);

CLUSTER aprs_lastobject_pkey ON aprs_lastobject;

--
-- aprs_laststatus
--
DROP TABLE aprs_laststatus;

CREATE TABLE aprs_laststatus (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  packet_date timestamp not null default now(),
  source varchar(20) NOT NULL default '',
  comment varchar(256),
  power int default NULL,
  height int default NULL,
  gain int default NULL,
  directivity int default NULL,
  rate int default NULL,
  symbol_table char(1) default NULL,
  symbol_code char(1) default NULL,
  PRIMARY KEY  (source)
);

CLUSTER aprs_laststatus_pkey ON aprs_laststatus;

--
-- aprs_lastweather
--
DROP TABLE aprs_lastweather;

CREATE TABLE aprs_lastweather (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  source varchar(20) NOT NULL default '',
  packet_date timestamp not null default now(),
  wind_direction int default NULL,
  wind_speed int default NULL,
  wind_gust int default NULL,
  wind_sustained int default NULL,
  temperature int default NULL,
  rain_hour float default NULL,
  rain_calendar_day float default NULL,
  rain_24hour_day float default NULL,
  humidity int default NULL,
  barometer real default NULL,
  luminosity int default NULL,
  PRIMARY KEY  (source)
);

CLUSTER aprs_lastweather_pkey ON aprs_lastweather;

--
-- aprs_messages
--
DROP INDEX aprs_messages_ts_idx;
DROP INDEX aprs_messages_source_idx;
DROP TABLE aprs_messages;

CREATE TABLE aprs_messages (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  source varchar(20) NOT NULL default '',
  packet_date timestamp not null default now(),
  addressee varchar(10) default NULL,
  message varchar(73) default NULL,
  id varchar(10) default NULL
--  ,PRIMARY KEY (packet_id)
);

CREATE INDEX aprs_messages_source_idx ON aprs_messages (source);
CREATE INDEX aprs_messages_ts_idx ON aprs_messages(packet_date);

--
-- aprs_msg2email_snoop_users
--
DROP INDEX aprs_msg2email_snoop_users_source_idx;
DROP TABLE aprs_msg2email_snoop_users;

CREATE TABLE aprs_msg2email_snoop_users (
  source varchar(20) NOT NULL default '',
  recipient varchar(20) NOT NULL,
  form varchar(32) NOT NULL default 'simple.frm'
);

CREATE INDEX aprs_msg2email_snoop_users_source_idx ON aprs_msg2email_snoop_users(source);

--
-- aprs_msg2email_users
--
DROP INDEX aprs_msg2email_users_source_idx;
DROP TABLE aprs_msg2email_users;

CREATE TABLE aprs_msg2email_users (
  source varchar(20) NOT NULL default '',
  email text NOT NULL,
  real_name varchar(32) NOT NULL default '',
  to_email text NOT NULL,
  PRIMARY KEY  (source)
);

CREATE INDEX aprs_msg2email_users_source_idx ON aprs_msg2email_users(source);

--
-- aprs_qa_hosts
--
DROP SEQUENCE aprs_qa_hosts_seq;
CREATE SEQUENCE aprs_qa_hosts_seq;

DROP TABLE aprs_qa_hosts;
CREATE TABLE aprs_qa_hosts (
  id int NOT NULL default nextval('aprs_qa_hosts_seq'),
  host varchar(255) NOT NULL default '',
  active boolean NOT NULL default '0',
  PRIMARY KEY  (id),
  CONSTRAINT aprs_qa_hosts_id_uniq_cnt UNIQUE (id)
);


--
-- aprs_raw
--
DROP INDEX aprs_raw_ts_idx;
DROP INDEX aprs_raw_source_idx;
DROP SEQUENCE aprs_raw_seq;
DROP TABLE aprs_raw;

CREATE SEQUENCE aprs_raw_seq;
CREATE TABLE aprs_raw (
  packet_id bigint NOT NULL DEFAULT nextval('aprs_raw_seq'),
  received_date TIMESTAMP NOT NULL DEFAULT now(),
  ip_source text default NULL,
  source varchar(20) NOT NULL default '',
  destination varchar(20) NOT NULL default '',
  digipeater_0 varchar(20) default NULL,
  digipeater_1 varchar(20) default NULL,
  digipeater_2 varchar(20) default NULL,
  digipeater_3 varchar(20) default NULL,
  digipeater_4 varchar(20) default NULL,
  digipeater_5 varchar(20) default NULL,
  digipeater_6 varchar(20) default NULL,
  digipeater_7 varchar(20) default NULL,
  information varchar(200),
  hostid int NOT NULL default '0',
  PRIMARY KEY  (packet_id)
--  CONSTRAINT aprs_raw_pid_uniq_cnt UNIQUE (packet_id)
);

CREATE INDEX aprs_raw_source_idx ON aprs_raw (source);
CREATE INDEX aprs_raw_ts_idx ON aprs_raw (received_date);

--
-- aprs_status
--
DROP INDEX aprs_status_source_idx;
DROP TABLE aprs_status;

CREATE TABLE aprs_status (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  packet_date timestamp not null default now(),
  source varchar(20) NOT NULL default '',
  comment varchar(256),
  power int default NULL,
  height int default NULL,
  gain int default NULL,
  directivity int default NULL,
  rate int default NULL,
  symbol_table char(1) default NULL,
  symbol_code char(1) default NULL
--  ,PRIMARY KEY  (packet_id)
);

CREATE INDEX aprs_status_source_idx ON aprs_status (source);

--
-- aprs_telemetry
--
DROP INDEX aprs_telemetry_ts_idx;
DROP INDEX aprs_telemetry_source_idx;
DROP TABLE aprs_telemetry;

CREATE TABLE aprs_telemetry (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  source varchar(20) NOT NULL default '',
  packet_date timestamp not null default now(),
  sequence int NOT NULL default '0',
  analog_0 int NOT NULL default '0',
  analog_1 int NOT NULL default '0',
  analog_2 int NOT NULL default '0',
  analog_3 int NOT NULL default '0',
  analog_4 int NOT NULL default '0',
  digital int NOT NULL default '0'
--  ,PRIMARY KEY  (packet_id)
);

CREATE INDEX aprs_telemetry_source_idx ON aprs_telemetry (source);
CREATE INDEX aprs_telemetry_ts_idx ON aprs_telemetry(packet_date);

--
-- aprs_telemetry_coefficients
--
--DROP INDEX aprs_telemetry_coefficients_ts_idx;
DROP INDEX aprs_telemetry_coefficients_source_idx;
DROP TABLE aprs_telemetry_coefficients;

CREATE TABLE aprs_telemetry_coefficients (
  source varchar(20) NOT NULL default '',
  packet_date timestamp not null default now(),
  a_0 real default NULL,
  b_0 real default NULL,
  c_0 real default NULL,
  a_1 real default NULL,
  b_1 real default NULL,
  c_1 real default NULL,
  a_2 real default NULL,
  b_2 real default NULL,
  c_2 real default NULL,
  a_3 real default NULL,
  b_3 real default NULL,
  c_3 real default NULL,
  a_4 real default NULL,
  b_4 real default NULL,
  c_4 real default NULL
);

CREATE INDEX aprs_telemetry_coefficients_source_idx ON aprs_telemetry_coefficients (source);
CREATE INDEX aprs_telemetry_coefficients_ts_idx ON aprs_telemetry_coefficients(packet_date);

--
-- aprs_telemetry_labels
--
DROP INDEX aprs_telemetry_labels_ts_idx;
DROP INDEX aprs_telemetry_labels_source_idx;
DROP TABLE aprs_telemetry_labels;

CREATE TABLE aprs_telemetry_labels (
  packet_date timestamp not null default now(),
  source varchar(20) NOT NULL default '',
  a_0 varchar(20) NOT NULL,
  a_1 varchar(20) NOT NULL,
  a_2 varchar(20) NOT NULL,
  a_3 varchar(20) NOT NULL,
  a_4 varchar(20) NOT NULL,
  d_0 varchar(20) NOT NULL,
  d_1 varchar(20) NOT NULL,
  d_2 varchar(20) NOT NULL,
  d_3 varchar(20) NOT NULL,
  d_4 varchar(20) NOT NULL,
  d_5 varchar(20) NOT NULL,
  d_6 varchar(20) NOT NULL,
  d_7 varchar(20) NOT NULL
);

CREATE INDEX aprs_telemetry_labels_source_idx ON aprs_telemetry_labels (source);
CREATE INDEX aprs_telemetry_labels_ts_idx ON aprs_telemetry_labels(packet_date);

--
-- aprs_telemetry_units
--
DROP INDEX aprs_telemetry_units_ts_idx;
DROP INDEX aprs_telemetry_units_source_idx;
DROP TABLE aprs_telemetry_units;

CREATE TABLE aprs_telemetry_units (
  source varchar(20) NOT NULL default '',
  packet_date timestamp not null default now(),
  a_0 varchar(128) NOT NULL,
  a_1 varchar(128) NOT NULL,
  a_2 varchar(128) NOT NULL,
  a_3 varchar(128) NOT NULL,
  a_4 varchar(128) NOT NULL,
  d_0 varchar(128) NOT NULL,
  d_1 varchar(128) NOT NULL,
  d_2 varchar(128) NOT NULL,
  d_3 varchar(128) NOT NULL,
  d_4 varchar(128) NOT NULL,
  d_5 varchar(128) NOT NULL,
  d_6 varchar(128) NOT NULL,
  d_7 varchar(128) NOT NULL
);

CREATE INDEX aprs_telemetry_units_source_idx ON aprs_telemetry_units (source);
CREATE INDEX aprs_telemetry_units_ts_idx ON aprs_telemetry_units(packet_date);

--
-- aprs_watch_programs
--
DROP TABLE aprs_watch_programs;
CREATE TABLE aprs_watch_programs (
  program int NOT NULL default '0',
  sys_user varchar(20) default NULL,
  directory text,
  command text NOT NULL,
  PRIMARY KEY  (program)
);


--
-- aprs_watch_users
--
DROP TABLE aprs_watch_users;
CREATE TABLE aprs_watch_users (
  source varchar(20) NOT NULL default '',
  program int NOT NULL default '0'
);


--
-- aprs_weather
--
DROP INDEX aprs_weather_source_idx;
DROP INDEX aprs_weather_ts_idx;
DROP TABLE aprs_weather;

CREATE TABLE aprs_weather (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  source varchar(20) NOT NULL default '',
  packet_date timestamp not null default now(),
  wind_direction int default NULL,
  wind_speed int default NULL,
  wind_gust int default NULL,
  wind_sustained int default NULL,
  temperature int default NULL,
  rain_hour float default NULL,
  rain_calendar_day float default NULL,
  rain_24hour_day float default NULL,
  humidity int default NULL,
  barometer real default NULL,
  luminosity int default NULL
--  ,PRIMARY KEY  (packet_id)
);

CREATE INDEX aprs_weather_source_idx ON aprs_weather (source);
CREATE INDEX aprs_weather_ts_idx ON aprs_weather(packet_date);

--
-- aprs_item
--

DROP INDEX aprs_item_ts_idx;
DROP INDEX aprs_item_source_idx;
--DROP INDEX aprs_item_geom_idx;
SELECT DropGeometryColumn('', 'aprs_item', 'position');
DROP TABLE aprs_item;

CREATE TABLE aprs_item (
  packet_id bigint NOT NULL default currval('aprs_raw_seq'),
  packet_date timestamp not null default now(),
  source varchar(20) NOT NULL,
  name varchar(20) NOT NULL,
  symbol_table char(1) default NULL,
  symbol_code char(1) default NULL,
  comment varchar(256)
--  ,PRIMARY KEY (packet_id)
);

SELECT AddGeometryColumn('', 'aprs_item', 'position', -1, 'POINT', 2 );
--CREATE INDEX aprs_item_geom_idx ON aprs_item USING GIST (position GIST_GEOMETRY_OPS);
CREATE INDEX aprs_item_source_idx ON aprs_item (source);
CREATE INDEX aprs_item_ts_idx ON aprs_item (packet_date);

--
-- aprs_lastitem
--

DROP INDEX aprs_lastitem_ts_idx;
DROP INDEX aprs_lastitem_source_idx;
--DROP INDEX aprs_lastitem_geom_idx;
SELECT DropGeometryColumn('', 'aprs_lastitem', 'position');
DROP TABLE aprs_lastitem;

CREATE TABLE aprs_lastitem (
  packet_id BIGINT NOT NULL default currval('aprs_raw_seq'),
  packet_date timestamp not null default now(),
  source varchar(20) NOT NULL,
  name varchar(20) NOT NULL,
  symbol_table char(1) default NULL,
  symbol_code char(1) default NULL,
  comment varchar(256),
  PRIMARY KEY (source, name)
);

SELECT AddGeometryColumn('', 'aprs_lastitem', 'position', -1, 'POINT', 2 );
--CREATE INDEX aprs_lastitem_geom_idx ON aprs_lastitem USING GIST (position GIST_GEOMETRY_OPS);
CREATE INDEX aprs_lastitem_source_idx ON aprs_lastitem (source);
CREATE INDEX aprs_lastitem_ts_idx ON aprs_lastitem (packet_date);

CLUSTER aprs_lastitem_pkey ON aprs_lastitem;


--
-- aprs_userdefined
--

DROP TABLE aprs_userdefined;
CREATE TABLE aprs_userdefined (
  packet_id BIGINT NOT NULL default currval('aprs_raw_seq'),
  source varchar(20) NOT NULL,
  packet_date timestamp NOT null default now(),
  user_id char(1) NOT NULL,
  packet_type char(1) NOT NULL,
  information varchar(256) NOT NULL,
  PRIMARY KEY (packet_id)
);


--
-- aprs_lastuserdefined
--

DROP TABLE aprs_lastuserdefined;
CREATE TABLE aprs_lastuserdefined (
  packet_id BIGINT NOT NULL default currval('aprs_raw_seq'),
  source varchar(20) NOT NULL,
  packet_date timestamp NOT null default now(),
  user_id char(1) NOT NULL,
  packet_type char(1) NOT NULL,
  information varchar(256) NOT NULL,
  PRIMARY KEY (source)
);


CREATE OR REPLACE FUNCTION aprs_distance(geometry, geometry) RETURNS FLOAT AS '
DECLARE
	var_a ALIAS FOR $1;
	var_b ALIAS FOR $2;
BEGIN

	return earth_distance(ll_to_earth(X(var_a), Y(var_a)), ll_to_earth(X(var_b), Y(var_b)));

END;
' LANGUAGE 'plpgsql';
